#include "obrazgl.h"
#include <QtGui>
#include <cstdio>
#include <cmath>
#include <QWheelEvent>

void obrazGL::mousePressEvent(QMouseEvent *event)
{
  mousePosition=event->pos();
}

void obrazGL::mouseMoveEvent(QMouseEvent *event)
{
  int dx=event->x()-mousePosition.x();
  int dy=event->y()-mousePosition.y();

  if (event->buttons() & Qt::LeftButton)
  {
    xRot+=dy;
    yRot+=dx;
    updateGL();
  }
  if (event->buttons() & Qt::RightButton)
  {
    yRot+=dx;
    zRot+=dy;
    updateGL();
  }
  mousePosition=event->pos();
}

void obrazGL::wheelEvent(QWheelEvent *event)
{
  zOdl+=float(event->delta())/120.0;
  if (zOdl>100.0) zOdl=100.0;
  if (zOdl<0) zOdl=0.0;
  updateGL();
}

void obrazGL::resizeGL(int width,int height)
{
  glViewport (0, 0, (GLsizei) width, (GLsizei) height); 
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective (30.0, 1.0, 0.01,100.0);
  glMatrixMode (GL_MODELVIEW);
}
void obrazGL::paintGL()
{
  glClear (GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
  glLoadIdentity ();             /* clear the matrix */
  /* viewing transformation  */
  gluLookAt (0.0, 0.0, -20.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
  while (xRot>360) xRot-=360.0;
  while (xRot<0) xRot+=360.0;
  while (yRot>360) yRot-=360.0;
  while (yRot<0) yRot+=360.0;
  while (zRot>360) zRot-=360.0;
  while (zRot<0) zRot+=360.0;
  glTranslated(0.0,0.0,zOdl);
  glRotatef((GLfloat) xRot,1.0,0.0,0.0);
  glRotatef((GLfloat) yRot,0.0,1.0,0.0);
  glRotatef((GLfloat) zRot,0.0,0.0,1.0);
  setBlackRubber();
  glLineWidth(6.0);
  glBegin(GL_LINE_STRIP);
  glVertex3d(0.0,0.0,0.0);
  glVertex3d(-10.0,0.0,0.0);
  glVertex3d(-10.0,-10.0,0.0);
  glVertex3d(-10.5,-10.0,0.0);
  glVertex3d(-10.0,-9.5,0.0);
  glVertex3d(-9.5,-10.0,0.0);
  glVertex3d(-10.0,-10.0,0.0);
  glEnd();
  setSilver();
  glBegin(GL_QUADS);
  glVertex3d(-20.0,-10.0,-20.0);
  glVertex3d(-20.0,-10.0,20.0);
  glVertex3d(20.0,-10.0,20.0);
  glVertex3d(20.0,-10.0,-20.0);
  glEnd();
  glPushMatrix();
  glRotatef(angle*180/3.1415926,0.0,0.0,1.0);
  setBronze();
  glLineWidth(2.0);
  glPushMatrix();
  glBegin(GL_LINES);
  glVertex3d(0.0,0.0,0.0);
  glVertex3d(0.0,-3.0,0.0);
  glEnd();
  glPopMatrix();
  glTranslated(0.0,-4.0,0.0);
  setEmerald();
  glCallList(sfera);
  glPopMatrix();
}
void obrazGL::setBronze()
{
  GLfloat mat_ambient[]={0.2125,0.1275,0.054};
  GLfloat mat_diffuse[]={0.714,0.4284,0.18144};
  GLfloat mat_specular[]={0.393548,0.271906,0.166721};
  GLfloat mat_shininess[]={0.2*128.0};
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambient);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
  glMaterialfv(GL_FRONT,GL_SHININESS,mat_shininess);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diffuse);
}
void obrazGL::setSilver()
{
  GLfloat mat_ambient[]={0.19225,0.19225,0.19225};
  GLfloat mat_diffuse[]={0.50754,0.50754,0.50754};
  GLfloat mat_specular[]={0.508273,0.508273,0.508273};
  GLfloat mat_shininess[]={0.4*128.0};
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambient);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
  glMaterialfv(GL_FRONT,GL_SHININESS,mat_shininess);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diffuse);
}
void obrazGL::setEmerald()
{
  GLfloat mat_ambient[]={0.0215,0.1745,0.0215};
  GLfloat mat_diffuse[]={0.07568,0.61424,0.07568};
  GLfloat mat_specular[]={0.633,0.727811,0.633};
  GLfloat mat_shininess[]={0.6*128.0};
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambient);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
  glMaterialfv(GL_FRONT,GL_SHININESS,mat_shininess);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diffuse);
}
void obrazGL::setBlackRubber()
{
  GLfloat mat_ambient[]={0.02,0.02,0.02};
  GLfloat mat_diffuse[]={0.01,0.01,0.01};
  GLfloat mat_specular[]={0.4,0.4,0.4};
  GLfloat mat_shininess[]={128.0*0.25};
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambient);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
  glMaterialfv(GL_FRONT,GL_SHININESS,mat_shininess);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diffuse);
}


void obrazGL::setDiffuse()
{
  GLfloat p[]={8.0,5.0,3.0,0.0};
  GLfloat l[]={0.6,0.6,0.6,0.0};
  glLightfv(GL_LIGHT2,GL_DIFFUSE,l);
  glLightfv(GL_LIGHT2,GL_POSITION,p);
  glEnable(GL_LIGHT2);
}
void obrazGL::setAmbient()
{
  GLfloat p[]={0.0,0.0,0.0,1.0};
  GLfloat l[]={0.3,0.3,0.3,0.0};
  glLightfv(GL_LIGHT0,GL_AMBIENT,l);
  glLightfv(GL_LIGHT0,GL_POSITION,p);
  glEnable(GL_LIGHT0);
}
void obrazGL::initializeGL()
{
  glClearColor(0.0,0.0,0.0,0.0);
  glShadeModel(GL_FLAT);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  sfera=glGenLists(1);
  glNewList(sfera,GL_COMPILE);
  GLfloat mat_ambient[]={0.2125,0.1275,0.054};
  GLfloat mat_diffuse[]={0.714,0.4284,0.18144};
  GLfloat mat_specular[]={0.393548,0.271906,0.166721};
  GLfloat mat_shininess[]={0.2*128.0};
  glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ambient);
  glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
  glMaterialfv(GL_FRONT,GL_SHININESS,mat_shininess);
  glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diffuse);
  renderSolidSphere(1.0,40,40);
  glEndList();
  setAmbient();
  setDiffuse();
}

obrazGL::obrazGL(QWidget* parent) : QGLWidget(parent)
{
  xRot=0;
  yRot=0;
  zRot=0;
  angle=0.0;
  anglespeed=0;
  zOdl=10.0;
  timer=new QTimer(this);
  connect(timer,SIGNAL(timeout()),this,SLOT(moveup()));
  timer->start(10);
  setMinimumWidth(400);
  setMinimumHeight(600);
}
void obrazGL::moveup()
{
  angle+=anglespeed*0.1;
  anglespeed-=sin(angle)*0.1;
  emit(anglechanged(angle,anglespeed));
  updateGL();
}
obrazGL::~obrazGL()
{
    makeCurrent();
}
void obrazGL::angleadjusted(float a,float b)
{
  angle=a;
  anglespeed=b;
  updateGL();
}
void obrazGL::renderSolidSphere(float r,int meridianDensity,int longitudeDensity)
{
  float xRS,yRS,zRS;
  float lg,mr;
  const float merSkip=3.1415926/float(meridianDensity);
  const float longSkip=2.0*3.1415926/float(longitudeDensity);
  const float merCen=merSkip/2.0;
  const float longCen=longSkip/2.0;
  lg=0;mr=0;
  glBegin(GL_QUADS);
  for(int i=0;i<meridianDensity; i++)
  {
    mr+=merSkip;
    for(int j=0;j<longitudeDensity;j++)
    {
      lg+=longSkip;
      xRS=r*cos(lg+longCen)*sin(mr+merCen);
      yRS=r*sin(mr+merCen)*sin(lg+longCen);
      zRS=r*cos(mr+merCen);
      glVertex3f(xRS,yRS,zRS);
      xRS=r*cos(lg+longCen)*sin(mr-merCen);
      yRS=r*sin(mr-merCen)*sin(lg+longCen);
      zRS=r*cos(mr-merCen);
      glVertex3f(xRS,yRS,zRS);
      xRS=r*cos(lg-longCen)*sin(mr-merCen);
      yRS=r*sin(mr-merCen)*sin(lg-longCen);
      zRS=r*cos(mr-merCen);
      glVertex3f(xRS,yRS,zRS);
      xRS=r*cos(lg-longCen)*sin(mr+merCen);
      yRS=r*sin(mr+merCen)*sin(lg-longCen);
      zRS=r*cos(mr+merCen);
      glVertex3f(xRS,yRS,zRS);
      xRS=r*cos(lg)*sin(mr);
      yRS=r*sin(mr)*sin(lg);
      zRS=r*cos(mr);
      glNormal3f(xRS,yRS,zRS);
    }
  } 
  glEnd();
}