#include "plansza.h"
#include <cmath>
using namespace std;
plansza::plansza(QWidget *parent): QGLWidget(parent)
{
  xcur=0.0;
  ycur=0.0;  
}

plansza::~plansza()
{
  makeCurrent();
}

void plansza::makeportrait() 
{
  GLuint portret;
  if (glIsList(portret)==GL_TRUE) glDeleteLists(portret,1);
  portret=glGenLists(1);
  float x,y;
  int j;
  glNewList(portret,GL_COMPILE);
  glColor3f(0.0,0.0,0.0);
  glBegin(GL_POINTS);
  for (int i=0; i<gestosc; i++)
  {
    y=ymax*(float(i)/float(gestosc))*0.5;
    x=0.0;
    j=0;
    do
    {
      x+=0.01*y;
      y-=0.01*sin(x);
      j++;
      glVertex2f(x,y);
      glVertex2f(x,-y);
      glVertex2f(-x,y);
      glVertex2f(-x,-y);
      glVertex2f(x+pi2,y);
      glVertex2f(-x+pi2,y);
      glVertex2f(x+pi2,-y);
      glVertex2f(-x+pi2,-y);
      glVertex2f(x-pi2,y);
      glVertex2f(-x-pi2,y);
      glVertex2f(x-pi2,-y);
      glVertex2f(-x-pi2,-y);
    }
    while ((y<0 || j<100 || x>0.01 || x<0.01) && (j<1000) && (x<pi2) && (x>-pi2));
  }
  glEnd();
  glEndList();
  if (glIsList(portrait)==GL_TRUE) glDeleteLists(portrait,1);
  portrait=portret;
  GLuint kolo=glGenLists(1);
  glNewList(kolo,GL_COMPILE);
  glColor3f(1.0,0.0,0.0);
  glBegin(GL_LINES);
  for (int i=0;i<360;i++)
  {
    glVertex2f(0.0,0.0);
    glVertex2f(radx*cos(double(i)*pi2/360.0),rady*sin(double(i)*pi2/360.0));
  }
  glEnd();
  glEndList();
  circle=kolo;
}

void plansza::resizeGL(int width,int height)
{
  glViewport (0, 0, (GLsizei) width, (GLsizei) height); 
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluOrtho2D(-0.5*xmax,0.5*xmax,-0.5*ymax,0.5*ymax);
  glMatrixMode (GL_MODELVIEW);
  wid=width;
  hei=height;
}
void plansza::initializeGL()
{
  makeportrait();
  glClearColor(1.0,1.0,1.0,0.0);
  glShadeModel(GL_FLAT);
  glDisable(GL_DEPTH_TEST); 
  glColor3f(1.0,1.0,1.0);
}
void plansza::paintGL()
{
  glClear(GL_COLOR_BUFFER_BIT);
  glLoadIdentity();
  glCallList(portrait);
  glColor3f(1.0,0.0,0.0);
  glColor3f(0.0,0.0,1.0);
  glBegin(GL_LINES);
  glVertex2f(-xmax,0.0); glVertex2f(xmax,0.0);
  glVertex2f(0.0,-ymax); glVertex2f(0.0,ymax);
  glEnd();
  glColor3f(1.0,0.0,0.0);
  glEnable(GL_LINE_STIPPLE);
  glLineStipple(2,0xAAAA);
  glBegin(GL_LINES);
  glVertex2f(0.5*pi2,-ymax); glVertex2f(0.5*pi2,ymax);
  glVertex2f(-0.5*pi2,-ymax); glVertex2f(-0.5*pi2,ymax);
  glEnd();
  glDisable(GL_LINE_STIPPLE);
  glTranslatef(xcur,ycur,0.0);
  glCallList(circle);
  
/*  glBegin(GL_QUADS);
  glVertex2f(xcur-0.1,ycur-0.1);
  glVertex2f(xcur-0.1,ycur+0.1);
  glVertex2f(xcur+0.1,ycur+0.1);
  glVertex2f(xcur+0.1,ycur-0.1);
  glEnd();*/
  
}

void plansza::mousePressEvent(QMouseEvent *event)
{
float dx=(float)event->x();
float dy=(float)event->y();
float w=0.5*(float)wid;
float h=0.5*(float)hei;
if (!(event->buttons() & Qt::LeftButton)) return;
xcur=0.5*(dx/w-1.0)*xmax;
ycur=-0.5*(dy/h-1.0)*ymax;
emit pointchosen(xcur,ycur);
}
void plansza::pointmoved(float a,float b)
{
  xcur=a;
  ycur=b;
  while (xcur>xmax) {xcur-=2*pi2;}
  while (xcur<-xmax) {xcur+=2*pi2;}
  updateGL();
}

